/***
 * Copyright (c) 2008-2016 CommonsWare, LLC
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not
 * use this file except in compliance with the License. You may
 * obtain	a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0.
 * Unless required
 * by applicable law or agreed to in writing, software distributed
 * under the
 * License is distributed on an "AS IS" BASIS,	WITHOUT	WARRANTIES OR
 * CONDITIONS
 * OF ANY KIND, either express or implied. See the License for the
 * specific
 * language governing permissions and limitations under the License.
 * <p/>
 * From _The Busy Coder's Guide to Android Development_
 * https://commonsware.com/Android
 */

package com.commonsware.android.notif.sound;

import android.Manifest;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ClipData;
import android.content.ContentResolver;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.StrictMode;
import android.support.v4.app.NotificationCompat;
import android.support.v4.content.FileProvider;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;
import java.io.File;

public class MainActivity extends AbstractPermissionActivity {
  private static final String AUTHORITY=
    BuildConfig.APPLICATION_ID+".provider";
  private static final String[] PERMS={
    Manifest.permission.WRITE_EXTERNAL_STORAGE
  };
  private File soundFile;
  private MenuItem allowFile;

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    getMenuInflater().inflate(R.menu.actions, menu);
    allowFile=menu.findItem(R.id.allow_file);

    return(super.onCreateOptionsMenu(menu));
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    if (item.getItemId()==R.id.allow_file) {
      StrictMode.setVmPolicy(new StrictMode.VmPolicy.Builder().build());
      allowFile.setEnabled(false);

      return(true);
    }

    return super.onOptionsItemSelected(item);
  }

  @Override
  protected String[] getDesiredPermissions() {
    return(PERMS);
  }

  @Override
  protected void handlePermissionDenied() {
    Toast
      .makeText(this, "Cannot proceed without permission", Toast.LENGTH_LONG)
      .show();
    finish();
  }

  @Override
  protected void onReady() {
    setContentView(R.layout.main);

    soundFile=
      new File(Environment.getExternalStorageDirectory(),
        "Notifications/clip.ogg");
  }

  public void playFile(View view) {
    raiseNotification(Uri.fromFile(soundFile), false);
  }

  public void playFileProvider(View view) {
    raiseNotification(FileProvider.getUriForFile(this, AUTHORITY,
      soundFile), false);
  }

  public void playWithClipData(View view) {
    raiseNotification(FileProvider.getUriForFile(this, AUTHORITY,
      soundFile), true);
  }

  public void playResource(View view) {
    raiseNotification(Uri.parse("android.resource://"+BuildConfig.APPLICATION_ID+"/"+R.raw.clip), false);
  }

  private void raiseNotification(Uri sound, boolean applyClipDataHack) {
    NotificationCompat.Builder b=new NotificationCompat.Builder(this);
    Intent outbound=
      new Intent(Intent.ACTION_VIEW,
        Uri.parse("https://commonsware.com"));

    if (applyClipDataHack) {
      outbound.setClipData(ClipData.newRawUri("foo", sound));
      outbound.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
    }

    PendingIntent pi=PendingIntent.getActivity(this, 0,
      outbound, PendingIntent.FLAG_UPDATE_CURRENT);

    b
      .setAutoCancel(true)
      .setSound(sound)
      .setContentTitle("A notification")
      .setContentText("Were you expecting something profound here?")
      .setSmallIcon(android.R.drawable.stat_sys_download_done)
      .setTicker("Hi!")
      .setContentIntent(pi);

    NotificationManager mgr=
      (NotificationManager)getSystemService(NOTIFICATION_SERVICE);

    mgr.notify(1337, b.build());
  }
}
